'use strict';

/*jshint unused: false*/
(function() {

  var app = angular.module('upsConsole', [
    'upsConsole.services',
    'ngResource',
    'ngNewRouter',
    'ngAnimate',
    'ngIdle',
    'ui.bootstrap',
    'ups.directives',
    'patternfly.notification'
  ]);

  /**
   * Snippet extracted from Keycloak examples
   */
  var auth = {};

  angular.element(document).ready(function () {
    var keycloak = new Keycloak('config/admin-ui-keycloak.json');
    auth.loggedIn = false;

    keycloak.init({ onLoad: 'login-required' }).success(function () {
      auth.loggedIn = true;
      auth.keycloak = keycloak;
      auth.logout = function() {
        auth.loggedIn = false;
        auth.keycloak = null;
        window.location = keycloak.authServerUrl + '/realms/aerogear/tokens/logout?redirect_uri=' + window.location.href;
      };
      app.factory('Auth', function () {
        return auth;
      });
      angular.bootstrap(document, ['upsConsole']);
    }).error(function () {
      window.location.reload();
    });

  });

  app.run(function($rootScope) {
    // allow to retrieve $rootScope in views (for clarification of access scope)
    $rootScope.$rootScope = $rootScope;
  });

  app.factory('Auth', function () {
    return auth;
  });

  var appConfig = {
    logDebugEnabled: false,
    idleDuration: 300,
    idleWarningDuration : 30,
    keepaliveInterval: 5
  };

  app.provider('appConfig', function () {
    return {
      set: function (settings) {
        // allow to override configuration (e.g. in tests)
        angular.extend(appConfig, settings);
      },
      $get: function () {
        // default configuration
        return appConfig;
      }
    };
  });

  app.factory('authInterceptor', function ($q, Auth) {
    return {
      request: function (config) {
        var deferred = $q.defer();

        if (config.url === 'rest/sender' || config.url === 'rest/registry/device/importer') {
          return config;
        }

        if (Auth.keycloak && Auth.keycloak.token) {
          Auth.keycloak.updateToken(5).success(function () {
            config.headers = config.headers || {};
            config.headers.Authorization = 'Bearer ' + Auth.keycloak.token;

            deferred.resolve(config);
          }).error(function () {
            window.location.reload();
          });
        }
        return deferred.promise;
      }
    };
  });

  app.config(function ($httpProvider) {
    $httpProvider.interceptors.push('authInterceptor');
  });

  app.config(function ($logProvider, appConfigProvider) {
    var appConfig = appConfigProvider.$get();
    $logProvider.debugEnabled( appConfig.logDebugEnabled );
  });


})();

'use strict';

angular.module('upsConsole')
  .config(function( $provide ) {

    /**
     * Decorator for $resource service which helps to maintain conciseness of the code by avoiding a need to call resource.method(...).$promise.then(...),
     * because it returns a promise directly: resource.method(...).then(...).
     */
    $provide.decorator('$resource', function($delegate) {
      return function decorator(url, paramDefaults, actions) {
        var args = Array.prototype.slice.call(arguments);
        var wrappedResource = {};
        var originalActions = {};
        var actionsWithoutFunctions = args[2] = {};
        Object.keys(actions).forEach(function( methodName ) {
          var method = actions[methodName];
          originalActions[methodName] = method;
          if (!angular.isFunction(method)) {
            actionsWithoutFunctions[methodName] = method;
          }
        });
        var originalResource = $delegate.apply($delegate, args);
        Object.keys(originalActions).forEach(function( methodName ) {
          var method = originalActions[methodName];
          if (angular.isFunction(method)) {
            wrappedResource[methodName] = method;
          } else {
            wrappedResource[methodName] = function() {
              return originalResource[methodName].apply(originalResource, arguments).$promise;
            };
          }
        });
        return wrappedResource;
      };
    });
  });


'use strict';

angular.module('upsConsole')
  .controller('RouteController', function($router) {
    $router.config([
      {path: '/',                     component: 'home'},
      {path: '/welcome',              component: 'welcome'},
      {path: '/wizard/create-app',    component: 'wizard01CreateApp'},
      {path: '/wizard/add-variant',   component: 'wizard02AddVariant'},
      {path: '/wizard/register-device',  component: 'wizard03RegisterDevice'},
      {path: '/wizard/send-push-notification',  component: 'wizard04SendPushNotification'},
      {path: '/wizard/setup-sender',  component: 'wizard05SetupSender'},
      {path: '/wizard/done',  component: 'wizard06Done'},
      {path: '/app/:app/:tab',        component: 'appDetail'},
    ]);
  });

'use strict';

angular.module('upsConsole')
  .controller('IdleController', function(Keepalive, Idle, $rootScope, $scope, $log, appConfig, Auth) {

    var self = this;

    self.config = appConfig;
    /**
     * idle service, keepalive, auth token refresh
     */
    Idle.watch();
    self.idleCountdown = appConfig.idleWarningDuration + 1;
    $rootScope.$on('KeepaliveResponse', function() {
      Auth.keycloak.updateToken(45).success(function(refreshed) {
        if (refreshed) {
          $log.debug('token was successfully refreshed');
        } else {
          $log.debug('token is still valid');
        }
      }).error(function() {
        $log.debug('failed to refresh the token, or the session has expired');
      });
    });

    $rootScope.$on('IdleStart', function() {
      $log.debug('idleStart');

    });
    $rootScope.$on('IdleWarn', function() {
      $log.debug('idleWarn');
      $scope.$apply(function() {
        self.idleCountdown = self.idleCountdown - 1;
      });
    });
    $rootScope.$on('IdleEnd', function() {
      $log.debug('idleEnd');
      $scope.$apply(function() {
        self.idleCountdown = appConfig.idleWarningDuration + 1;
      });

    });
    $rootScope.$on('IdleTimeout', function() {
      $log.debug('idleTimeout');
      Auth.logout();
    });
  })

  .config( function( KeepaliveProvider, IdleProvider, appConfigProvider ) {
    var appConfig = appConfigProvider.$get();
    IdleProvider.idle( appConfig.idleDuration );
    IdleProvider.timeout( appConfig.idleWarningDuration );
    KeepaliveProvider.interval( appConfig.keepaliveInterval );
  });

/*
 * JBoss, Home of Professional Open Source
 * Copyright Red Hat, Inc., and individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';

angular.module('upsConsole')
  .controller('AppController', function ( $rootScope, $scope, Auth, $http, $interval, $timeout, $log, appConfig, dashboardEndpoint ) {

    var self = this;

    this.config = appConfig;
    this.warnings = [];

    //Retrieve the current logged in username
    function getUsername() {
      return Auth.keycloak.idTokenParsed.preferred_username;
    }
    this.username = getUsername();
    $scope.$watch(getUsername, function( newValue ) {
      self.username = newValue;
    });

    this.logout = function() {
      Auth.logout();
    };

    this.goToAccountManagement = function() {
      window.location = Auth.keycloak.authServerUrl + '/realms/aerogear/account?referrer=unified-push-server-js';
    };

    this.havePendingRequests = function() {
      return $http.pendingRequests.some(function(config) {
        if (config.method !== 'GET') {
          return true;
        }
      });
    };


    this.stats = {};
    function updateStats() {
      dashboardEndpoint.totals()
        .then(function( data ) {
          self.stats = data;
        });
    }
    updateStats();
    $scope.$on('upsUpdateStats', updateStats);
    $scope.$on('upsApplicationCreated', updateStats);
    $scope.$on('upsApplicationDeleted', updateStats);

    // load warnings and update them periodically and when notification is sent
    function updateWarnings() {
      dashboardEndpoint.warnings().then(function( warnings ) {
        self.warnings = warnings;
      });
    }
    updateWarnings();
    $interval(updateWarnings, 30000);
    $scope.$on('upsNotificationSent', function() {
      $timeout(updateWarnings, 1500);
      $timeout(updateWarnings, 5000);
      $timeout(updateWarnings, 10000);
    });
  });

angular.module('upsConsole')
  .controller('HomeController', function( $q, $modal, $router, $rootScope, applicationsEndpoint, dashboardEndpoint ) {

    var self = this;

    this.apps = [];
    this.topNotifications = [];

    this.canActivate = function() {
      this.currentPage = 1;
      return self.fetchNewPage(1)
        .then(function() {
          if (self.totalItems < 1) {
            $router.parent.navigate('/welcome');
            return false;
          }
        });
    };

    this.activate = function() {
      dashboardEndpoint.latestActiveApps()
        .then(function( data ) {
          self.topNotifications = data;
        });
    };

    this.pageChanged = function(page) {
      self.fetchNewPage(page);
    };

    this.fetchNewPage = function(page) {
      return applicationsEndpoint.fetch(page)
        .then(function( result ) {
          self.apps = result.apps;
          self.totalItems = result.totalItems;
        });
    };

    this.deleteApp = function(app) {
      $modal.open({
        templateUrl: 'views/dialogs/remove-app.html',
        controller: function( $modalInstance, $scope ) {
          $scope.app = app;
          $scope.confirm = function() {
            applicationsEndpoint.delete({appId: app.pushApplicationID})
              .then(function () {
                $rootScope.$broadcast('upsApplicationDeleted');
                return self.fetchNewPage(self.currentPage);
              })
              .then(function() {
                $modalInstance.close();
                if (self.totalItems < 1) {
                  $router.parent.navigate('/welcome');
                }
              });
          };
          $scope.dismiss = function() {
            $modalInstance.dismiss('cancel');
          }
        }
      });
    };

  });

angular.module('upsConsole')
  .controller('WelcomeController', function( $rootScope ) {

    this.activate = function() {
      $rootScope.$broadcast('upsUpdateStats');
    };

  });

angular.module('upsConsole')
  .controller('AppDetailController', function( $rootScope, $q, $routeParams, $modal, applicationsEndpoint, messageSenderEndpoint, metricsEndpoint, ContextProvider, Notifications ) {

    var self = this;

    this.app = null; // is retrieved in canActivate
    this.notifications = null; // is retrieved in canActivate
    this.tab = $routeParams.tab;


    this.contextPath = ContextProvider.contextPath();

    this.canActivate = function() {
      return $q.all([
        applicationsEndpoint.getWithMetrics({appId: $routeParams.app})
          .then(function( app ) {
            self.app = app;
            if ( !app.variants.length ) {
              self.tab = 'variants';
            }
          }),
        metricsEndpoint.fetchApplicationMetrics($routeParams.app, 1)
          .then(function( data ) {
            self.notifications = data.pushMetrics;
          })
      ]);
    };

    this.sendNotification = function() {
      $modal.open({
        templateUrl: 'views/dialogs/send-push-notification.html',
        controller: function( $scope, $modalInstance ) {

          $scope.app = self.app;

          // default message
          $scope.pushData = {
            'message': {
              'sound': 'default',
              'alert': '',
              'simple-push': 'version=' + new Date().getTime()
            },
            'criteria' : {}
          };

          $scope.send = function() {
            if ($scope.selectedVariants) {
              $scope.pushData.criteria.variants = $scope.selectedVariants;
            }
            if($scope.aliases) {
              $scope.pushData.criteria.alias = $scope.aliases.split(',');
            }
            if($scope.deviceTypes) {
              $scope.pushData.criteria.deviceType = $scope.deviceTypes.split(',');
            }
            if($scope.categories) {
              $scope.pushData.criteria.categories = $scope.categories.split(',');
            }

            messageSenderEndpoint( self.app.pushApplicationID, self.app.masterSecret ).send({}, $scope.pushData)
              .then(function() {
                self.app.$messageCount += 1;
                self.notifications.unshift({ submitDate: new Date().getTime() });
                $modalInstance.close();
                $rootScope.$broadcast('upsNotificationSent', $scope.pushData, $scope.app);
                Notifications.success('Notification was successfully sent');
              })
              .catch(function() {
                Notifications.error('Failed to sent notification');
              });
          };

          $scope.cancel = function() {
            $modalInstance.dismiss();
          };
        }
      });
    };

  });

angular.module('upsConsole')
  .controller('SenderController', function( $modal, $scope, applicationsEndpoint, ContextProvider ) {

    var self = this;

    this.app = $scope.$parent.$parent.appDetail.app;

    this.contextPath = ContextProvider.contextPath();

    this.renewMasterSecret = function () {
      $modal.open({
        templateUrl: 'inline:renew-master-secret.html',
        controller: function( $scope, $modalInstance ) {
          $scope.confirm = function() {
            applicationsEndpoint.reset({appId: self.app.pushApplicationID}, function (application) {
              self.app.masterSecret = application.masterSecret;
              $modalInstance.close( application );
            });
          };
          $scope.dismiss = function() {
            $modalInstance.dismiss('cancel');
          }
        }
      });
    };
  });

angular.module('upsConsole')
  .controller('VariantsController', function ( $modal, variantModal, $scope, variantsEndpoint, Notifications ) {

    var self = this;

    this.app = $scope.$parent.$parent.appDetail.app;

    /* split the variant types to the groups so that they can be easily access */
    function splitByType( variants ) {
      return variants
        .sort(function(a, b) {
          return a.type.localeCompare(b.type);
        })
        .reduce(function(variantList, variant) {
          var type = variant.type.match(/^windows/) ? 'windows' : variant.type;
          var variantType = variantList[type] = variantList[type] || [];
          variantType.push(variant);
          variantType.$deviceCount = (variantType.$deviceCount  || 0) + (variant.$deviceCount || 0);
          variantType.$messageCount = (variantType.$messageCount  || 0) + (variant.$messageCount || 0);
          return variantList;
        }, {});
    }
    this.byType = splitByType( this.app.variants );

    this.add = function() {
      return variantModal.add( this.app )
        .then(function( variant ) {
          variant.$deviceCount = 0;
          variant.$messageCount = 0;
          self.app.variants.push( variant );
          self.byType = splitByType( self.app.variants );
          Notifications.success('Variant ' + variant.name + ' successfully created');
        })
        .catch(function() {
          Notifications.error('Failed to create variant ' + variant.name);
        });
    };

    this.edit = function( variant ) {
      var variantEditableCopy = angular.extend({}, variant)
      return variantModal.edit( this.app, variantEditableCopy )
        .then(function( variant ) {
          var variantToUpdate = self.app.variants.filter(function(v) {
            return v.variantID === variant.variantID;
          })[0];
          angular.extend(variantToUpdate, variant);
          Notifications.success('Variant ' + variant.name + ' was successfully modified');
        })
        .catch(function() {
          Notifications.error('Failed to create variant ' + variant.name);
        });
    };

    this.delete = function( variant ) {
      $modal.open({
        templateUrl: 'views/dialogs/remove-variant.html',
        controller: function( $modalInstance, $scope ) {
          $scope.variant = variant;
          $scope.confirm = function() {
            variantsEndpoint.delete({
                  appId: self.app.pushApplicationID,
                  variantType: variant.type,
                  variantId: variant.variantID })
              .then(function () {
                self.app.variants = self.app.variants.filter(function( v ) {
                  return v != variant;
                });
                self.byType = splitByType( self.app.variants );
                $modalInstance.close();
              });
          };
          $scope.dismiss = function() {
            $modalInstance.dismiss('cancel');
          }
        }
      });
    };

  });

angular.module('upsConsole')
  .controller('ActivityController', function ( $modal, variantModal, $scope, metricsEndpoint ) {

    var self = this;

    this.app = $scope.$parent.$parent.appDetail.app;
    this.metrics = [];
    this.totalCount = 0;
    this.currentPage = 1;
    this.currentStart = 0;
    this.currentEnd = 0;
    this.perPage = 10;

    function fetchMetricsPage( page ) {
      metricsEndpoint.fetchApplicationMetrics(self.app.pushApplicationID, page, self.perPage)
        .then(function( data ) {
          self.metrics = data.pushMetrics;
          self.totalCount = data.totalItems;
          self.currentStart = self.perPage * (self.currentPage - 1) + 1;
          self.currentEnd = self.perPage * (self.currentPage - 1) + self.metrics.length;
          self.metrics.forEach(function( metric ) {
            try {
              metric.$message = JSON.parse(metric.rawJsonMessage);
            } catch (err) {
              console.log('failed to parse metric')
              metric.$message = {};
            }
          });
        });
    }

    // initial page
    fetchMetricsPage( 1 );

    this.onPageChange = function ( page ) {
      fetchMetricsPage( page );
    };

    $scope.$on('upsNotificationSent', function( pushData, app ) {
      fetchMetricsPage( self.currentPage );
    });

  });

angular.module('upsConsole')
  .controller('Wizard01CreateAppController', function ( $rootScope, variantModal, $router, applicationsEndpoint, createAppWizard, Notifications ) {

    var self = this;

    this.application =  {};

    this.createApp = function() {
      applicationsEndpoint.create( self.application )
        .then(function( app ) {
          createAppWizard.app = app;
          Notifications.success('Application ' + self.application.name + ' successfully created');
          $rootScope.$broadcast('upsUpdateStats');
          $router.root.navigate('/wizard/add-variant');
        })
        .catch(function() {
          Notifications.error('Failed to create application ' + self.application.name);
        });
    }
  })

  .factory('createAppWizard', function() {
    return {
      app: null,
      variant: null
    };
  });

angular.module('upsConsole')
  .controller('Wizard02AddVariantController', function( variantModal, $router, createAppWizard, Notifications ) {

    this.canActivate = function() {
      console.log('canActivate');
      if ( createAppWizard.app ) {
        return true;
      } else {
        $router.root.navigate('/wizard/create-app');
        return false;
      }
    };

    this.app = createAppWizard.app;

    this.addVariant = function() {
      return variantModal.add( this.app )
        .then(function( variant ) {
          createAppWizard.variant = variant;
          Notifications.success('Variant ' + variant.name + ' successfully created');
          $router.root.navigate('/wizard/register-device');
        })
        .catch(function() {
          Notifications.error('Failed to create variant ' + variant.name);
        });
    };

  });



angular.module('upsConsole')
  .controller('Wizard03RegisterDeviceController', function( variantModal, $router, createAppWizard ) {

    this.canActivate = function() {
      console.log('canActivate');
      if ( !createAppWizard.app ) {
        $router.root.navigate('/wizard/create-app');
        return false;
      }
      if ( !createAppWizard.variant ) {
        $router.root.navigate('/wizard/add-variant');
        return false;
      }
      return true;
    };

    this.app = createAppWizard.app;
    this.variant = createAppWizard.variant;

    this.addVariant = function() {
      return variantModal.add()
        .then(function( variant ) {
          console.log('success');
          console.log(variant);
          createAppWizard.variant = variant;
          $router.root.navigate('/wizard/register-device');
        })
        .catch(function() {
          console.log('error');
        })
    };

  });



angular.module('upsConsole')
  .controller('Wizard04SendPushNotificationController', function( $router, createAppWizard, Notifications, $rootScope, messageSenderEndpoint ) {

    var self = this;

    this.canActivate = function() {
      if ( !createAppWizard.app ) {
        $router.root.navigate('/wizard/create-app');
        return false;
      }
      if ( !createAppWizard.variant ) {
        $router.root.navigate('/wizard/add-variant');
        return false;
      }
      return true;
    };

    this.app = createAppWizard.app;
    this.variant = createAppWizard.variant;

    this.pushData = {
      'message': {
        'sound': 'default',
        'alert': 'Hello! This is my first notification to ' + (self.variant ? self.variant.name : null),
        'simple-push': 'version=' + new Date().getTime()
      },
      'criteria' : {}
    };

    this.sendNotification = function() {
      messageSenderEndpoint( self.app.pushApplicationID, self.app.masterSecret ).send({}, self.pushData)
        .then(function() {
          $rootScope.$broadcast('upsNotificationSent', self.pushData, self.app);
          Notifications.success('Notification was successfully sent');
          $router.root.navigate('/wizard/setup-sender');
        })
        .catch(function() {
          Notifications.error('Failed to sent notification');
        });
    };

  });



angular.module('upsConsole')
  .controller('Wizard05SetupSenderController', function( variantModal, $router, createAppWizard, ContextProvider ) {

    var self = this;

    this.canActivate = function() {
      if ( !createAppWizard.app ) {
        $router.root.navigate('/wizard/create-app');
        return false;
      }
      if ( !createAppWizard.variant ) {
        $router.root.navigate('/wizard/add-variant');
        return false;
      }
      return true;
    };

    this.app = createAppWizard.app;
    this.variant = createAppWizard.variant;
    this.contextPath = ContextProvider.contextPath();

  });



'use strict';

angular.module('upsConsole.services', []).
  value('version', '0.1');
'use strict';

angular.module('upsConsole.services').factory('variantModal', function ($modal, $q, variantsEndpoint) {
  var service = {
    add: function (app) {
      return $modal.open({
        templateUrl: 'views/dialogs/create-variant.html',
        controller: function ($scope, $modalInstance) {

          $scope.isNew = true;
          $scope.variant = {}; // start with empty variant
          $scope.variant.certificates = []; // initialize file list for upload

          $scope.confirm = function () {
            var variantData = extractValidVariantData($scope.variant);

            var createFunction = (variantData instanceof FormData) ? variantsEndpoint.createWithFormData : variantsEndpoint.create;

            createFunction({
              appId: app.pushApplicationID,
              variantType: extractVariantType($scope.variant)
            }, variantData)
              .then(function (variant) {
                $modalInstance.close(variant);
              })
              .catch(function (err) {
                $modalInstance.dismiss(err);
              });
          };

          $scope.dismiss = function () {
            $modalInstance.dismiss('cancel');
          };

          $scope.validateFileInputs = function () {
            switch ($scope.variant.type) {
            case 'ios':
              return $scope.variant.certificates.length > 0;
            }
            return true;
          };
        }
      }).result;
    },

    edit: function (app, variant) {
      return $modal.open({
        templateUrl: 'views/dialogs/create-variant.html',
        controller: function ($scope, $modalInstance) {

          $scope.isNew = false;
          $scope.variant = variant;
          $scope.variant.certificates = []; // initialize file list for upload

          $scope.confirm = function () {
            var endpointParams = {
              appId: app.pushApplicationID,
              variantType: extractVariantType($scope.variant),
              variantId: variant.variantID
            };
            var variantData = extractValidVariantData($scope.variant);
            var promise;
            if (variant.type !== 'ios') {
              promise = variantsEndpoint.update(endpointParams, variantData);
            } else {
              if (variant.certificates.length > 0) {
                promise = variantsEndpoint.updateWithFormData(endpointParams, variantData);
              } else {
                promise = variantsEndpoint.patch(endpointParams, {
                  name: variant.name,
                  description: variant.description,
                  production: variant.production
                });
              }
            }
            promise.then(function (modifiedVariant) {
              modifiedVariant.variantID = variant.variantID;
              $modalInstance.close(modifiedVariant);
            });
          };

          $scope.dismiss = function () {
            $modalInstance.dismiss('cancel');
          };

          $scope.validateFileInputs = function () {
            if ($scope.variant.type === 'ios') {
              if (!$scope.variant.id) {
                // for new variant
                return $scope.variant.certificates.length > 0 && !!variant.passphrase; // we must enter certificate and passphrase
              } else {
                // for existing variant...
                if ($scope.variant.certificates.length > 0) { // if there is a certificate selected
                  return !!variant.passphrase; // we must provide a passphase
                }
              }
            }
            return true;
          };
        }
      }).result;
    }
  };

  function extractValidVariantData(variant) {
    var properties = ['name'], result = {};
    switch (variant.type) {
    case 'android':
      properties = properties.concat(['projectNumber', 'googleKey']);
      break;
    case 'simplePush':
      properties = properties.concat([]);
      break;
    case 'ios':
      if (variant.certificates && variant.certificates.length) {
        variant.certificate = variant.certificates[0];
      }
      properties = properties.concat(['production', 'passphrase', 'certificate']);
      var formData = new FormData();
      properties.forEach(function (property) {
        formData.append(property, variant[property] === undefined ? '' : variant[property]);
      });
      return formData;
    case 'windows':
      if (variant.protocolType === 'wns') {
        properties = properties.concat(['sid', 'clientSecret', 'protocolType']);
      } else {
        properties = properties.concat(['protocolType']);
      }
      break;
    case 'windows_wns':
      result.protocolType = 'wns';
      properties = properties.concat(['sid', 'clientSecret']);
      break;
    case 'windows_mpns':
      result.protocolType = 'mpns';
      properties = properties.concat([]);
      break;
    case 'adm':
      properties = properties.concat(['clientId', 'clientSecret']);
      break;
    default:
      throw 'Unknown variant type ' + variant.type;
    }
    properties.forEach(function (property) {
      result[property] = variant[property];
    });
    return result;
  }

  function extractVariantType( variant ) {
    switch(variant.type) {
    case 'windows':
      return 'windows_' + variant.protocolType;
    default:
      return variant.type;
    }
  }

  return service;

});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('applicationsEndpoint', function ($resource, $q) {
  return $resource('rest/applications/:appId/:verb', {
    appId: '@appId'
  }, {
    get: {
      method: 'GET',
      params: {
        includeDeviceCount: true,
        includeActivity: true
      }
    },
    list: {
      method: 'GET',
      isArray: true,
      params: {
        includeDeviceCount: true,
        includeActivity: true
      }
    },
    create: {
      method: 'POST'
    },
    update: {
      method: 'PUT'
    },
    delete: {
      method: 'DELETE'
    },
    count: {
      method: 'GET',
      params: {verb: 'count'}
    },
    reset: {
      method: 'PUT',
      params: {verb: 'reset'}
    },
    fetch: function(pageNo) {
      var deferred = $q.defer();
      this.list({page: pageNo - 1, per_page: 8}, function (apps, responseHeaders) {
        var result = {
          apps: apps,
          totalItems: responseHeaders('total')
        };
        apps.forEach(function(app) {
          app.$messageCount = parseInt(responseHeaders('activity_app_' + app.pushApplicationID), 10);
          app.$deviceCount = parseInt(responseHeaders('deviceCount_app_' + app.pushApplicationID), 10);
        });
        deferred.resolve( result );
      });
      return deferred.promise;
    },
    getWithMetrics: function(params) {
      var deferred = $q.defer();
      this.get(params, function (app, responseHeaders) {
        app.$messageCount = parseInt(responseHeaders('activity_app_' + app.pushApplicationID), 10);
        app.$deviceCount = parseInt(responseHeaders('deviceCount_app_' + app.pushApplicationID), 10);
        app.variants.forEach(function( variant ) {
          variant.$messageCount = parseInt(responseHeaders('activity_variant_' + variant.variantID), 10);
          variant.$deviceCount = parseInt(responseHeaders('deviceCount_variant_' + variant.variantID), 10);
        });
        deferred.resolve( app );
      });
      return deferred.promise;
    }
  });
});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('variantsEndpoint', function ($resource) {
  return $resource('rest/applications/:appId/:variantType/:variantId/:verb', {
    appId: '@appId',
    variantType: '@variantType',
    variantId: '@variantId'
  }, {
    get: {
      method: 'GET'
    },
    query: {method: 'GET', isArray: true},
    create: {
      method: 'POST'
    },
    update: {
      method: 'PUT'
    },
    delete: {
      method: 'DELETE'
    },
    patch: {
      method: 'PATCH'
    },
    createWithFormData: {
      method: 'POST',
      headers: {'Content-Type': undefined},
      withCredentials: true,
      transformRequest: angular.identity
    },
    updateWithFormData: {
      method: 'PUT',
      headers: {'Content-Type': undefined},
      withCredentials: true,
      transformRequest: angular.identity
    },
    reset: {
      method: 'PUT',
      params: {verb: 'reset'}
    }
  });
});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('messageSenderEndpoint', function ($resource) {
  return function ( applicationID, masterSecret ) {
    return $resource('rest/sender', {}, {
      send: {
        method: 'POST',
        headers: {
          'aerogear-sender': 'AeroGear UnifiedPush Console',
          'Authorization': 'Basic ' + btoa(applicationID + ':' + masterSecret)
        }
      }
    });
  };
});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('metricsEndpoint', function ($resource, $q) {
  return $resource('rest/metrics/messages/:verb/:id', { id: '@id' }, {
    application: {
      method: 'GET',
      isArray: true,
      params: {
        verb: 'application'
      }
    },
    variant: {
      method: 'GET',
      isArray: true,
      params: {
        verb: 'variant'
      }
    },
    fetchVariantMetrics: function(variantId, pageNo) {
      var deferred = $q.defer();
      this.variant({id: variantId, page: pageNo - 1, per_page: 10, sort:'desc'}, function (data, responseHeaders) {
        angular.forEach(data, function (metric) {
          metric.totalReceivers = metric.variantInformations[0].receivers;
          metric.deliveryFailed = !metric.variantInformations[0].deliveryStatus;
        });
        deferred.resolve({
          totalItems: responseHeaders('total'),
          pushMetrics: data
        });
      });
      return deferred.promise;
    },
    fetchApplicationMetrics: function(applicationId, pageNo, perPage) {
      perPage = perPage || 10;
      var deferred = $q.defer();
      this.application({id: applicationId, page: pageNo - 1, per_page: perPage, sort:'desc'}, function (data, responseHeaders) {
        angular.forEach(data, function (metric) {
          angular.forEach(metric.variantInformations, function (variant) {
            if (!variant.deliveryStatus) {
              metric.deliveryFailed = true;
            }
          });
        });
        deferred.resolve({
          totalItems: responseHeaders('total'),
          pushMetrics: data
        });
      });
      return deferred.promise;
    }
  });
});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('dashboardEndpoint', function ($resource) {
  return $resource('rest/metrics/dashboard/:verb', {}, {
    totals: {
      method: 'GET'
    },
    warnings: {
      method: 'GET',
      isArray: true,
      params: {
        verb: 'warnings'
      }
    },
    latestActiveApps: {
      method: 'GET',
      isArray: true,
      params: {
        verb: 'active',
        count: 4
      }
    }
  });
});

'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.factory('ContextProvider', function($location) {
  return {
    /**
     * getter for the root context of the application
     * (the URL where application can be accessed)
     */
    contextPath: function() {
      var href = $location.absUrl();
      href = href.split('#', 1).join('');
      href = href.split('?', 1).join('');
      return href;
    }
  };
});
'use strict';

var upsServices = angular.module('upsConsole.services');

upsServices.config(function($httpProvider) {
  $httpProvider.interceptors.push(function($q, $interpolate, Notifications) {
    return {
      'responseError': function(rejection) {
        var messageExpr, message;
        if (rejection.status === 400 && typeof rejection.data === 'object') {
          Object.keys(rejection.data).forEach( function( key ) {
            message = rejection.data[key];
            Notifications.error( message );
          });
        } else {
          messageExpr = $interpolate('Server returned {{status}}: {{statusText}}');
          message = messageExpr( rejection );
          Notifications.error(messageExpr(rejection));
        }
        return $q.reject(rejection);
      }
    };
  });
});

'use strict';

/* Directives */
angular.module('ups.directives', ['upsConsole.services'])

  .directive('upsNavigation', function () {
    return {
      scope: {
        current: '@'
      },
      restrict: 'E',
      replace: true,
      templateUrl: 'directives/ups-navigation.html'
    };
  })

  .directive('upsBreadcrumb', function () {
    return {
      templateUrl: 'directives/ups-breadcrumb.html',
      controller: function($scope, $compile, breadcrumbs) {
        $scope.breadcrumbs = breadcrumbs;
      }
    };
  })

  .directive('upsFiles', function () {
    return {
      scope: {
        'files': '=upsFiles',
        'onChange': '&onChange'
      },
      restrict: 'A',
      replace: false,
      link: function ($scope, $element) {
        $element.bind('change', function (e) {
          $scope.$apply(function() {
            while ($scope.files.length > 0) {
              $scope.files.pop();
            }
            for (var i in e.target.files) {
              if (typeof e.target.files[i] === 'object') {
                $scope.files.push(e.target.files[i]);
              }
            }
            $scope.onChange();
          });
        });
      }
    };
  })

  .directive('upsPluralize', function () {
    return {
      scope: {
        'noun': '@',
        'count': '=',
        'zero': '@'
      },
      restrict: 'E',
      template:
      '<span class="ups-pluralize" ng-if="count > 0"><span class="count">{{ count }}</span> {{ noun }}{{ count > 1 ? "s" : "" }}</span>' +
      '<span class="ups-pluralize zero" ng-if="count == 0"><span class="count">{{ zero == null ? "0" : zero }}</span> {{ noun }}s</span>'
    };
  })

  .directive('upsClientSnippets', function () {
    return {
      scope: {
        variant: '=',
        activeSnippet: '@'
      },
      controller: function( $scope, ContextProvider ) {
        $scope.typeEnum = {
          android:      { name: 'Android',    snippets: ['android', 'cordova'] },
          ios:          { name: 'iOS',        snippets: ['ios-objc', 'ios-swift', 'cordova']},
          windows_mpns: { name: 'Windows',    snippets: ['mpns', 'cordova'] },
          windows_wns:  { name: 'Windows',    snippets: ['wns', 'cordova'] },
          simplePush:   { name: 'SimplePush', snippets: ['cordova'] },
          adm:          { name: 'ADM',        snippets: ['cordova'] }
        };

        $scope.contextPath = ContextProvider.contextPath();

        $scope.state = {
          activeSnippet: $scope.activeSnippet
        };

        $scope.cordovaVariantType = (function() {
          switch ($scope.variant.type) {
          case 'windows_mpns':
            return 'windows';
          default:
            return $scope.variant.type;
          }
        })();

        $scope.senderID = $scope.variant.type === 'android' ? $scope.variant.projectNumber : null;
      },
      restrict: 'E',
      templateUrl: 'directives/ups-client-snippets.html'
    };
  })

  .directive('upsSenderSnippets', function () {
    return {
      scope: {
        app: '=',
        activeSnippet: '@'
      },
      controller: function( $scope, ContextProvider ) {
        $scope.contextPath = ContextProvider.contextPath();
      },
      restrict: 'E',
      templateUrl: 'directives/ups-sender-snippets.html'
    };
  });
